package org.greenstone.gsdl3.sql.derby;

import  org.greenstone.gsdl3.sql.*;


import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.greenstone.gsdl3.util.DBInfo;

import org.apache.log4j.*;

public class DerbyDBWrapper implements MetadataDBWrapper{
    protected SQLServer sqlServer = null;
    protected SQLStatements sqlState = null;
    protected Connection connection = null; 
    protected Statement statement = null;
    protected static Logger logger = Logger.getLogger(org.greenstone.gsdl3.sql.derby.DerbyDBWrapper.class.getName());


    public DBInfo getInfo(String id){
	if(sqlState == null) return null;
        String dbInfoState = sqlState.getDBInfoStatement(id);
        ArrayList result = executeQuery(dbInfoState);
	if (result.size() == 0) return null;
	DBInfo info = new DBInfo();
	for(int i=0;  i<result.size();i++){
	    HashMap arow = (HashMap)result.get(i);
	    Iterator ite = arow.keySet().iterator();
	    while(ite.hasNext()){
		String key = (String)ite.next();
		Object value = arow.get(key);
		//logger.info(key + "=>" +value);
		if (value == null || value.equals("")) continue;
		info.addInfo(key.toLowerCase(),sqlState.undoDBSafe(value));
	    }
	}

	return info;
    }

    public void setSQLServer(SQLServer server){
	sqlServer = server;
    }

    public void setSQLStatements(SQLStatements state){
	sqlState = state;
    }

    public boolean openConnection(String databasepath){
	connection = sqlServer.connect(databasepath);
	
	if (connection == null) return false;

	try{
		connection.setAutoCommit(false);	
		statement = connection.createStatement();
	}
	catch(SQLException sqle){
	    logger.error("Database Error occured when creating a statement object",sqle);
	    return false;
	}
	return true;
    }

    //return a list of rows array->hashmap
    public synchronized ArrayList executeQuery(String query_statement){
	//the database hasn't been correct yet
    	ArrayList results = new ArrayList();
    try{	
    	
    	if (statement == null){
    		logger.info("the database hasn't been correct yet");
    		return new ArrayList();
    	}
	
    	ResultSet rs = statement.executeQuery(query_statement);
    	ResultSetMetaData rsmd = rs.getMetaData();
    	int numOfColumns = rsmd.getColumnCount();
    	while(rs.next()){
	       HashMap arow = new HashMap();
	       for(int i = 1; i <= numOfColumns ; i++){
		   arow.put(rsmd.getColumnName(i).toLowerCase(), rs.getObject(i));
	       }
	       results.add(arow);  
	   }
	}
	catch(SQLException sqle){
	    logger.error("Database Error occured when executeQuery " + query_statement,sqle);
	    return results;
	}
	return results;
    }
    
     /** check for the existence of a table**/
    public boolean check4Table (String check_table) {
         try {
	  statement.execute(check_table);
      }  catch (SQLException sqle) {
	  String theError = (sqle).getSQLState();
	  if (theError.equals("42X05"))   // Table does not exist
	      { 
		  return false;
	      }   
	  else{
	      return true;
	  }
      }
      //  System.out.println("Just got the warning - table exists OK ");
      return true;
    }   

    public void closeConnection(String databasepath){
	sqlServer.disconnect(databasepath);
    }
  
 }